/*
    LLCryptoLib Examples
    Advanced .NET Encryption and Hashing Library
    
	 
    Copyright (C) 2003-2008 LittleLite Software
    
*/

using System;
using System.IO;

using LLCryptoLib.Utils;
using LLCryptoLib.Crypto;

namespace LLCryptoLibExamples
{
    /// <summary>
    /// Example of symmetric stream encryption
    /// </summary>
	public static class StreamEncryptionExample
	{

        /// <summary>
        /// Encrypts the file.
        /// </summary>
        public static void EncryptFile(SupportedStreamAlgorithms algo)
        {
        	FileInfo rndFile = RandomFile.CreateRandomBinary();
            StreamEncryptionExample.EncryptFile(rndFile, algo);
            StreamEncryptionExample.Cleanup(rndFile.FullName);
        }
        
        public static void EncryptFiles(SupportedStreamAlgorithms algo)
        {
        	const int fileLenBase = 134006;
        	const string fileNameBase = "testBinary.bin";
        			
        	for (int j=0; j<8; j++)
        	{
        		string fileName = fileNameBase;
        		FileInfo rndFile = RandomFile.CreateRandomBinary(fileName, fileLenBase+j);
        		StreamEncryptionExample.EncryptFile(rndFile, algo);
        		StreamEncryptionExample.Cleanup(rndFile.FullName);
        	}
     
        }
        
        public static void Cleanup(string fileNameBase)
        {
            // Clean up example files
        	if (File.Exists(fileNameBase))
        	{
        		File.Delete(fileNameBase);
        	}
        	if (File.Exists(fileNameBase+"_encoded.bin"))
        	{
        		File.Delete(fileNameBase+"_encoded.bin");
        	}
        	if (File.Exists(fileNameBase+"_encoded.bin_decoded.bin"))
        	{
        		File.Delete(fileNameBase+"_encoded.bin_decoded.bin");
        	}
        }

        /// <summary>
        /// Encrypts the file.
        /// </summary>
        /// <param name="algorithm">The algorithm.</param>
        private static void EncryptFile(FileInfo rndFile, SupportedStreamAlgorithms algorithm)
        {
            // 1. Create random file
            Console.WriteLine("File created.");
            string encryptedFile = rndFile.FullName + "_encoded.bin";
            string decryptedFile = encryptedFile + "_decoded.bin";

            // 2. Set algorithm
            IStreamAlgorithm cryptoAlgo = StreamAlgorithmFactory.Create(algorithm);

            // 3. Encrypt 
            StreamCrypter crypter = new StreamCrypter(cryptoAlgo);
            Console.WriteLine("Encrypting with "+crypter.Algorithm.ToString());
            crypter.GenerateKeys("littlelitesoftware");
            crypter.EncryptDecrypt(rndFile.FullName, encryptedFile, true, null);
            Console.WriteLine("File encrypted into " + encryptedFile);

            // 4. Decrypt
            StreamCrypter decrypter = new StreamCrypter(cryptoAlgo);
            decrypter.GenerateKeys("littlelitesoftware");
            decrypter.EncryptDecrypt(encryptedFile, decryptedFile, false, null);
            Console.WriteLine("File decrypted into " + decryptedFile);

            // 5. Confront them
            FileManager fm = FileManager.Reference;
            byte[] originalFileContents = fm.ReadBinaryFile(rndFile.FullName);
            byte[] decryptedFileContents = fm.ReadBinaryFile(decryptedFile);

            string fmFeedback = fm.ErrorMessage;
            if (fmFeedback.Length == 0)
            {
                if (originalFileContents.Length == decryptedFileContents.Length)
                {
                    bool areEqual = true;
                    int count = 0;
                    foreach (byte b in originalFileContents)
                    {
                        if (b != decryptedFileContents[count++])
                        {
                            areEqual = false;
                            Console.WriteLine("Found difference at byte " + count);
                            break;
                        }
                    }

                    if (areEqual)
                    {
                        Console.WriteLine("File decryption OK");
                    }
                    else
                    {
                        Console.WriteLine(">>> File decryption KO: same size but different bytes");
                    }
                }
                else
                {
                    FileInfo encryptedFileInfo = new FileInfo(encryptedFile);
                    Console.WriteLine(">>> File decryption KO: different lengths");
                    Console.WriteLine(">>> Original is " + originalFileContents.Length + " bytes.");
                    Console.WriteLine(">>> Encrypted is " + encryptedFileInfo.Length + " bytes.");
                    Console.WriteLine(">>> Decrypted is " + decryptedFileContents.Length + " bytes.");
                }
            }
            else
            {
                Console.WriteLine("IOError: "+fmFeedback);
            }

        }
	}
}
